<?php
/**
 * @version    $Id$
 * @package    JSN_PageBuilder3
 * @author     JoomlaShine Team <support@joomlashine.com>
 * @copyright  Copyright (C) 2012 JoomlaShine.com. All Rights Reserved.
 * @license    GNU/GPL v2 or later http://www.gnu.org/licenses/gpl-2.0.html
 *
 * Websites: http://www.joomlashine.com
 * Technical Support:  Feedback - http://www.joomlashine.com/contact-us/get-support.html
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

/**
 * JSN PageBuilder3 system plugin.
 *
 * @package  JSN_PageBuilder3
 * @since    1.0.0
 */
class PlgSystemPageBuilder3 extends JPlugin
{
	/**
	 * Define PageBuilder signatures.
	 *
	 * @var  string
	 * @since 1.0.0
	 */
	public static $start_html = '<!-- Start PageFly HTML -->';
	public static $end_html = '<!-- End PageFly HTML -->';
	public static $start_data = '<!-- Start PageFly Data|';
	public static $end_data = '|End PageFly Data -->';
	/**
	 * The PageBuilder 3 component.
	 *
	 * @var  string
	 * @since 1.0.0
	 */
	protected static $ext = 'com_pagebuilder3';
	/**
	 * Joomla application object.
	 *
	 * @var  object
	 * @since 1.0.0
	 */
	protected $app;

	/**
	 * Register onAfterInitialise event handler.
	 *
	 * @return  void
	 * @since 1.0.0
	 */
	public function onAfterInitialise()
	{
		$this->app = JFactory::getApplication();
		// Get active language.
		$lang = JFactory::getLanguage();

		// Check if language file exists for active language.
		if (!file_exists(JPATH_ROOT . '/administrator/language/' . $lang->get('tag') . '/' . $lang->get('tag') . '.plg_system_pagebuilder3.ini'))
		{
			// Load language file from plugin directory.
			$lang->load('plg_system_pagebuilder3', dirname(__FILE__), null, true);
		}
		else
		{
			$lang->load('plg_system_pagebuilder3', JPATH_ADMINISTRATOR, null, true);
		}

		// Initialize Ajax handler for PageBuilder app.
		require_once dirname(__FILE__) . '/includes/ajax.php';

		// Override Joomla's JEditor class declaration.
		JLoader::register('JEditor', dirname(__FILE__) . '/includes/editor.php');
	}

	/**
	 * Register onBeforeRender event handler.
	 *
	 * @return  void
	 * @since 1.0.0
	 */
	public function onBeforeRender()
	{
		$doc = JFactory::getDocument();
		// Generate button to switch editor.
		if (defined('JSNPB3_EDITOR_AVAILABLE'))
		{

			$doc->setMetaData('cache-control', 'no-cache', 'http-equiv');
			$doc->setMetaData('expires', '0', 'http-equiv');
			$doc->setMetaData('pragma', 'no-cache', 'http-equiv');
			JSNMenuHelper::addEntry(
				'jsn-pb3-editor-switcher',   // Button ID.
				JText::_('SWITCH_EDITOR'), // Button text.
				'javascript:void(0)',        // Button link.
				false,                       // Active state.
				'',                          // Button icon.
				'toolbar',                   // Button parent.
				'pb3-editor-switcher hidden' // Button class.
			);

			// Get active editor.
			$active = JFactory::getConfig()->get('editor');
			// Get list of available editors.
			$editors = JPluginHelper::getPlugin('editors');
			if (count($editors) > 1)
			{
				foreach ($editors as $editor)
				{
					if ($editor->name != $active)
					{
						// Prepare button text.
						$text = JText::_('SWITCH_TO_' . strtoupper($editor->name));

						if (0 === strpos($text, 'SWITCH_TO_'))
						{
							$text = ucfirst($editor->name);
						}

						JSNMenuHelper::addEntry(
							"switch-to-{$editor->name}", // Button ID.
							$text,                       // Button text.
							"#{$editor->name}",          // Button link.
							false,                       // Active state.
							'',                          // Button icon.
							'jsn-pb3-editor-switcher',   // Button parent.
							''                           // Button class.
						);
					}
				}
			}
			// Load assets for editor switcher.

			if($active == 'pagebuilder3') {
				JSNHtmlHelper::loadEditionManager('com_pagebuilder3', JUri::root() . '/plugins/system/pagebuilder3/assets/edition.js');
			}
			$doc->addStyleSheet(JUri::root(true) . '/plugins/system/pagebuilder3/assets/editor-switcher.css');


			$jversion   = new JVersion();
			$ver        = $jversion->getShortVersion();
			$isJoomla37 = version_compare($ver, '3.7', '>=');

			$doc->addScriptDeclaration('
			window.pagefly_data = window.pb2_editor_data = window.pb_editor_data = ' . json_encode(array(
					'baseURL'        => JUri::root(),
					'current_editor' => $active,
					'isNewContent'   => $this->isNewContent(),
					'page_id'        => $this->getPageId(),
					'token'          => JSession::getFormToken(),
					'isSite'         => $this->app->isSite(),
					'component'      => $this->app->input->getString('option'), 'isJoomla37' => $isJoomla37,
					'isOldContent'   => $this->isOldContent()
				)) . ';');



		}
		if (defined('JSNPB3_EDITOR_AVAILABLE') || defined('JSNPB2_EDITOR_AVAILABLE')) {

			$doc->addScript(JUri::root(true) . '/plugins/system/pagebuilder3/assets/editor-switcher.js');
			if ($this->isOldContent())
			{
				if ($active === 'pagebuilder3')
				{
					$this->app->enqueueMessage('This page was created by JSN PageBuilder 2, and it can be convert to JSN PageBuilder 3 data, if you really want to convert, please click Edit and Save this article!', 'notice');
				}
				else
				{
					$this->app->enqueueMessage('This page was created by JSN PageBuilder 2, and it can be convert to JSN PageBuilder 3 data, if you really want to convert, <a href="#" id="pb-switch-pagebuilder3" class="btn btn-small btn-primary"> click here</a> to switch to JSN PageBuilder 3', 'notice');
				}

			}
		}


		if ($this->app->isSite())
		{
			$doc->addStyleSheet(JUri::root(true) . '/plugins/editors/pagebuilder3/assets/app/main.css');		
			JHTML::_('behavior.modal');
		}
		$doc->addScriptDeclaration('
		window.pb_baseUrl = "' . JUri::root() . '";
		');

		$doc->addScript(JUri::root(true) . '/plugins/system/pagebuilder3/assets/custom.js');
	}

	/**
	 * Register onAfterRender event handler.
	 *
	 * @return  void
	 * @since 1.0.0
	 */
	public function onAfterRender()
	{
		// Render PageBuilder content if has.
		try
		{
			if ($this->app->isSite())
			{

				// Get the current response body.
				$body = $this->app->getBody();
				self::renderPageBuilderContent($body);
				$this->app->setBody($body);

			}
		}
		catch (Exception $e)
		{
			throw new $e;
		}
	}


	/**
	 * Method to render PageBuilder content.
	 *
	 * @param   string &$content Content to render.
	 *
	 * @return  void
	 * @since 1.0.0
	 */
	protected static function renderPageBuilderContent(&$content)
	{
		// Parse PageBuilder content.
		if (false !== strpos($content, self::$start_html) && false !== strpos($content, self::$end_html))
		{

			// Look for PageBuilder content (at deepest level first) based on defined signatures.
			$temp = explode(self::$end_html, $content, 2);
			$from = strpos($temp[0], self::$start_html);
			$html = substr($temp[0], $from + strlen(self::$start_html));
			// get the base64 data
			$fromData = strpos($temp[1], self::$start_data) + strlen(self::$start_data);
			$data     = substr($temp[1], $fromData, strpos($temp[1], self::$end_data) - $fromData);
			// Render PageBuilder data.
			$page = json_decode(base64_decode($data));
			$temp = $html;
			// Render all {pb3loadmodule} tags that are not rendered.
			$temp = self::renderModule($temp);
			if ($page)
			{
				$temp = self::renderDynamicContent($page, $temp);
			}
			// Re-build content.
			$content = str_replace(
				self::$start_html . $html . self::$end_html . self::$start_data . $data . self::$end_data,
				$temp,
				$content
			);
			$content = self::handleLink($content);
			// Continue render remaining PageBuilder content.
			self::renderPageBuilderContent($content);
		}
	}

	/**
	 * @param $page
	 * @param $temp
	 * @param $isNew boolean is new pagebuilder version (1.4.0)
	 *
	 * @return mixed
	 *
	 * @since 1.4.0
	 */
	private static function renderDynamicContent($page, $temp)
	{
		// Load PageBuilder helper class.
		if (!class_exists('JSNPageBuilder3ContentHelper'))
		{
			require_once JPATH_ROOT . '/administrator/components/' . self::$ext . '/helpers/content.php';
		}
		$attr = array();

		foreach ($page->items as $id => $element)
		{
			if (isset($element->url))
			{
				$url = $element->url;
				$url = preg_match('/^http/is', $url) ? $url : JUri::base() . $url;
				try
				{
					$res = JSNPageBuilder3ContentHelper::fetchHttp($url);
					if ($res === false)
					{
						$url = str_replace(JUri::root(), $_SERVER['HTTP_X_FORWARDED_BY'] . JUri::base(true) . '/', $url);
						$res = JSNPageBuilder3ContentHelper::fetchHttp($url);
					}
					if ($res !== false)
					{
						$attr[$id] = json_decode($res);
					}

				}
				catch (Exception $e)
				{
				}
			}
		}

		if (count($attr) > 0)
		{

			// Load Mustache template engine.
			if (!class_exists('Mustache_Engine'))
			{
				require_once JPATH_ROOT . '/administrator/components/' . self::$ext . '/libraries/3rd-party/mustache/mustache.php';
			}
			$mustache = new Mustache_Engine;

			$temp = $mustache->render($temp, $attr);
		}

		return $temp;
	}

	private static function renderModule($temp)
	{
		// Instantiate PB3 Load Module plugin.
		if (class_exists('PlgContentPB3LoadModule') || JPluginHelper::importPlugin('content', 'pb3loadmodule', false))
		{
			// Get Joomla event dispatcher.
			$dispatcher = JEventDispatcher::getInstance();

			// Load the plugin from the database.
			$plugin = JPluginHelper::getPlugin('content', 'pb3loadmodule');

			// Instantiate the plugin.
			$plugin = new PlgContentPB3LoadModule($dispatcher, ( array ) $plugin);

			// Emulate an article.
			$article = ( object ) array('text' => $temp);
			$params  = array();

			// Trigger onContentPrepare event to let the PB3 Load Module plugin render all {pb3loadmodule} tags.
			$dispatcher->trigger('onContentPrepare', array('', &$article, &$params, 0));

			return $article->text;
		}

		return $temp;
	}

	private static function handleLink($content)
	{

		$base      = JUri::base();
		$path_base = preg_replace("(^https?:)", "", JUri::base());
		// Replace all relative links with absolute URLs in HTML tags.
		if (preg_match_all('/(href|src)="([^"]+)"/', $content, $matches, PREG_SET_ORDER))
		{
			foreach ($matches as $match)
			{
				// Fix some wrong image url in k2
				if (strpos($match[0], 'src="' . $path_base . 'http') !== false)
				{
					preg_match('/(https?:\/\/[^\s]+)/', $match[0], $text);
					$content = str_replace($match[0], 'src="' . $text[0] . '"', $content);
				}

				//Fix broken image link.
				if (strpos($match[0], $base) > 0 && strpos($match[0], 'src="') !== false)
				{
					$content = str_replace($match[0], 'src="' . substr($match[0], strpos($match[0], $base)), $content);
				}
				// If link is relative, convert it to absolute.
				if (preg_match('/&pb-slug=1/', $match[2]))
				{
					$slugLink = str_replace('&pb-slug=1', '', $match[2]);
					$content  = str_replace($match[2], $slugLink, $content);
				}
				else if (!preg_match('/^([a-zA-Z0-9]+:|\/|#)/', $match[2]) && strpos($match[2], '{{{') === false)
				{
					$match[2] = "{$base}{$match[2]}";
					$content  = str_replace($match[0], $match[1] . '="' . $match[2] . '"', $content);
				}
			}
		}

		unset($matches);
		/**
		 * Replace all relative links with absolute URLs in CSS rules.
		 */
		if (preg_match_all('/url\(\s*[\'"]*([^\)]+)[\'"]*\s*\)/', $content, $matches, PREG_SET_ORDER))
		{
			foreach ($matches as $match)
			{
				// If link is relative, convert it to absolute.
				$match[1] = trim($match[1], '\'"');
				if (!preg_match('/^([a-zA-Z0-9]+:|\/|#)/', $match[1]))
				{
					$newUrl  = "{$base}{$match[1]}";
					$newUrl  = 'url(' . $newUrl . ')';
					$content = str_replace($match[0], $newUrl, $content);
				}
			}
		}

		return $content;

	}

	private function getContent()
	{

		$option = $this->app->input->getString('option', '');
		// check whether user is editing from non content/module edit, we don't support it yet
		if ($this->getPageId() === null || $option == 'com_categories')
		{
			return '';
		}
		$dbo  = JFactory::getDbo();
		$type = $option == 'com_modules' ? 'modules' : ($option == 'com_content' ? 'content' : '');
		if ($type === '')
		{
			return '';
		}
		$target = $type == 'modules' ? 'content' : 'introtext';
		$query  = $dbo->getQuery(true);
		$query
			->select($dbo->quoteName($target))
			->from($dbo->quoteName("#__$type"))
			->where($dbo->quoteName('id') . ' = ' . $this->getPageId());
		$dbo->setQuery($query);

		return $content = $dbo->loadResult();
	}

	/**
	 * Check Article is new content or not.
	 * @return bool
	 * @since 1.1.0
	 */
	private function isNewContent()
	{
		$content = $this->getContent();

		return strlen((string) $content) < 2;
	}

	/**
	 * get the article id
	 * @return int | null
	 * @since 1.1.0
	 */
	private function getPageId()
	{
		$jinput = JFactory::getApplication()->input;

		return $jinput->getString('id') !== '' ? $jinput->getString('id') : ($jinput->getString('cid') !== '' ? $jinput->getString('cid') : ($jinput->getString('a_id') !== '' ? $jinput->getString('a_id') : ($jinput->getString('virtuemart_product_id') ? $jinput->getString('virtuemart_product_id') : null)));
	}


	private function isOldContent()
	{
		$content = $this->getContent();

		return strpos($content, '<!-- Start PageBuilder Data|') !== false || strpos($content, '<!-- Start New PageBuilder Data|') !== false;
	}

}
