<?php
/**
 * @version    $Id$
 * @package    SUN Framework
 * @author     JoomlaShine Team <support@joomlashine.com>
 * @copyright  Copyright (C) 2012 JoomlaShine.com. All Rights Reserved.
 * @license    GNU/GPL v2 or later http://www.gnu.org/licenses/gpl-2.0.html
 *
 * Websites: http://www.joomlashine.com
 * Technical Support:  Feedback - http://www.joomlashine.com/contact-us/get-support.html
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

/**
 * Handle Ajax requests from token key pane.
 *
 * @package  SUN Framework
 * @since    1.0.0
 */
class SunFwAjaxTokenKey extends SunFwAjax
{
	/**
	 * Get token key data.
	 *
	 * @param   boolean  $return  Whether to return data or send response back immediately?
	 *
	 * @return  mixed
	 */
	public function getAction($return = false)
	{
		// Get update token.
		$pluginParams = SunFwHelper::getExtensionParams('plugin', 'sunfw', 'system');

		/**
		 * Prepare response data.
		 */
		$data = array(
			'url' => $this->baseUrl,
			'token' => empty($pluginParams['token']) ? '' : $pluginParams['token'],
			'description' => JText::_('SUNFW_TOKEN_KEY_DESC'),
			'usernameLabel' => JText::_('SUNFW_USERNAME'),
			'passwordLabel' => JText::_('SUNFW_PASSWORD'),
			'getTokenLabel' => JText::_('SUNFW_GET_TOKEN_KEY'),
			'currentTokenLabel' => JText::_('SUNFW_CURRENT_TOKEN_KEY'),
			'tokenInputPlaceholder' => JText::_('SUNFW_TOKEN_KEY_IS_NOT_SET'),
			'verifyLabel' => JText::_('SUNFW_FRAMEWORK_VERIFY')
		);

		if ($return)
		{
			return $data;
		}

		$this->setResponse($data);
	}

	/**
	 * Get token key.
	 */
	public function getTokenKeyAction()
	{
		// Check token.
		if ( ! JSession::checkToken('get') )
		{
			throw new Exception('Invalid Token');
		}

		// Check permission.
		if ( ! JFactory::getUser()->authorise('core.admin') )
		{
			throw new Exception( JText::_('JGLOBAL_AUTH_ACCESS_DENIED') );
		}

		// Get customer information.
		$method   = $this->input->getMethod();
		$username = $this->input->getUsername('username', '');
		$password = $this->input->$method->get('password', '', 'RAW');

		if ($username == '' || $password == '')
		{
			throw new Exception( JText::_('SUNFW_LIGHTCART_ERROR_TOKEN_ERR01') );
		}

		// Prepare data.
		$randCode = SunFwUltils::generateRandString();
		$domain   = JURI::root();

		preg_match('@^(?:http://www\.|http://|www\.|http:|https://www\.|https://|www\.|https:)?([^/]+)@i', $domain, $domainFilter);

		$domain    = $domainFilter[1];
		$secretKey = md5($randCode . $domain);
		$query     = array();

		$query['rand_code' ] = $randCode;
		$query['domain'    ] = $domain;
		$query['secret_key'] = $secretKey;
		$query['username'  ] = $username;
		$query['password'  ] = $password;

		// Get token key.
		try
		{
			$http = new JHttp;
			$data = $http->post(SUNFW_GET_TOKEN_URL, $query);

			// JSON-decode the result.
			$result = json_decode($data->body);

			if ( is_null($result) )
			{
				throw new Exception( JText::_('SUNFW_ERROR_FAILED_TO_CONNECT_OUR_SERVER') );
			}

			if ( (string) $result->result == 'error' )
			{
				throw new Exception( JText::_('SUNFW_LIGHTCART_ERROR_' . $result->message) );
			}

			// Store token key.
			SunFwHelper::updateExtensionParams(
				array(
					'token' => $result->token,
				),
				'plugin',
				'sunfw',
				'system'
			);

			// Send client information to JSN server.
			require_once JPATH_ROOT . '/plugins/system/sunfw/includes/client/client.php';

			try
			{
				SunFwClientInformation::postClientInformation($result->token);
			}
			catch (Exception $e)
			{
				// Do nothing.
			}

			$this->setResponse( array(
				'message' => JText::_('SUNFW_TOKEN_IS_VALID'),
				'token' => $result->token
			) );
		}
		catch (Exception $e)
		{
			throw $e;
		}
	}

	/**
	 * Clear token key.
	 *
	 * @return  void
	 */
	public function clearTokenKeyAction()
	{
		// Check token.
		if ( ! JSession::checkToken('get') )
		{
			throw new Exception('Invalid Token');
		}

		// Check permission.
		if ( ! JFactory::getUser()->authorise('core.admin') )
		{
			throw new Exception( JText::_('JGLOBAL_AUTH_ACCESS_DENIED') );
		}

		// Clear token key.
		SunFwHelper::updateExtensionParams(
			array(
				'token' => '',
			),
			'plugin',
			'sunfw',
			'system'
		);
	}
}
